%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Ke,Re,Ie] =trian_2d(e_mat,e_spa,i_var,mat)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% stiffness matrix Ke and righthand side Re for linear tets
% i_var:  [rho^0-rho_0^*]/rho_0^*    relativ change in reference density
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[nmat,ngrw] = size(mat);
 nod  = 3;
 ndim = 2;
delta = eye(ndim);

Ie = i_var;
Re = zeros(6,1);
Ke = zeros(6,6);

indx  =[1;3;5];                       % indices of x coordinates
ex_mat=e_mat(indx);                   % material x coordinates of 1,2,3
ex_spa=e_spa(indx);                   % spatial  x coordinates of 1,2,3

indy  =[2;4;6];                       % indices of y coordinates
ey_mat=e_mat(indy);                   % material y coordinates of 1,2,3 
ey_spa=e_spa(indy);                   % spatial  y coordinates of 1,2,3
 
% integration point %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gp(1)=(ex_mat(1)+ex_mat(2)+ex_mat(3)) / 3.0; 
gp(2)=(ey_mat(1)+ey_mat(2)+ey_mat(3)) / 3.0; 

Ae =(ex_mat(1)*ey_mat(2) + ex_mat(2)*ey_mat(3) + ex_mat(3)*ey_mat(1) ...
   - ex_mat(2)*ey_mat(1) - ex_mat(3)*ey_mat(2) - ex_mat(1)*ey_mat(3))/2.0;
if (Ae <= 0) ; disp('alert!!!'); end
w  = Ae;

% shape functions in isoparametric space 
N(1)=1-gp(1)-gp(2);  
N(2)= +gp(1); 
N(3)=       +gp(2); 

% partial derivatives of shape functions wrt r and s
dNr(1,1)=-1;  dNr(1,2)=+1;  dNr(1,3)= 0;
dNr(2,1)=-1;  dNr(2,2)= 0;  dNr(2,3)=+1;

% jacobian transformation matrix
JT=dNr*[ex_mat;ey_mat]';
detJ=det(JT);
if detJ<10*eps;    disp('Jacobi determinant less than zero!');   end;
JTinv=inv(JT);
dNx=JTinv*dNr;

F  = zeros(2,2);
for i=1:3
  indx=[2*i-1; 2*i];
  F = F+e_spa(indx)'*dNx(:,i)';
end

var = i_var(1); 
if (ngrw==7); [A,P,var]=cnst_den(F,var,mat,ndim); end;  % density
if (ngrw==9); [A,P,var]=cnst_vol(F,var,mat,ndim); end;  % volume
Ie(1) = var;

for i=1:nod
  en=(i-1)*2;
  Re(en+ 1) =  Re(en+ 1) +(P(1,1)*dNx(1,i)' ...
						 + P(1,2)*dNx(2,i)') * detJ * w;
  Re(en+ 2) =  Re(en+ 2) +(P(2,1)*dNx(1,i)' ...
						 + P(2,2)*dNx(2,i)') * detJ * w;
end  

for i=1:nod
for j=1:nod
  eni=(i-1)*2;
  enj=(j-1)*2;
  for r=1:ndim
  for s=1:ndim
    Ke(eni+r,enj+s)=Ke(eni+r,enj+s) ...
	+(dNx(1,i)*(A(r,1,s,1)*dNx(1,j)+A(r,1,s,2)*dNx(2,j)) ...
	 +dNx(2,i)*(A(r,2,s,1)*dNx(1,j)+A(r,2,s,2)*dNx(2,j)))*detJ*w;
  end
  end
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


